import * as path from 'path';
import * as url from 'url';

import { PlatformUtil } from '@ucap/electron-core';
import { ChatChannel } from '@ucap/electron-native';
import {
  PlatformApi,
  IpcMainApi,
  BrowserWindowApi,
  AppApi
} from '@ucap/electron-common';
import { ChatRoomWindow } from '../windows/chat-room.window';

const rootPath = PlatformUtil.rootPath(__dirname).split(path.sep).join('/');
const distPath = path.join(rootPath, 'dist').split(path.sep).join('/');

export class AppChatService
  implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'appChatService';

  constructor(private configuration: AppApi.AppConfiguration) {}

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @IpcMainApi.Handle(ChatChannel.openRoom)
  async handleOpenRoom(
    event: Electron.IpcMainEvent,
    openUrl: string
  ): Promise<void> {
    const win = BrowserWindowApi.Builder.build(
      this.configuration,
      ChatRoomWindow
    );

    win.native.loadURL(
      __DEV__
        ? 'http://localhost:4200'
        : url.format({
            protocol: 'file:',
            slashes: true,
            pathname: path.resolve(distPath, 'index.html')
          })
    );
  }
}
