import * as Electron from 'electron';
import log from 'electron-log';

import { WindowIdle } from '@ucap/native';

import { BrowserWindowUtil } from '@ucap/electron-core';
import { IpcMainApi, PlatformApi } from '@ucap/electron-common';
import { IdleChannel } from '@ucap/electron-native';

export class AppIdleService
  implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'appIdleService';

  private limitTimeInSecond: number;
  private idle: WindowIdle;
  private handle: any;

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @IpcMainApi.Handle(IdleChannel.started)
  async handleStarted(event: Electron.IpcMainEvent): Promise<void> {
    log.debug('handleStarted');
  }

  @IpcMainApi.Handle(IdleChannel.startCheck)
  async handleStartCheck(
    event: Electron.IpcMainEvent,
    limitTime: number
  ): Promise<void> {
    log.debug('handleStartCheck');
    const __this = this;

    if (!!this.handle) {
      return;
    }

    this.limitTimeInSecond = limitTime;
    this.handle = setInterval(() => {
      __this.idleCheck();
    }, 1000 * 10);
  }

  @IpcMainApi.Handle(IdleChannel.stopCheck)
  async handleStopCheck(event: Electron.IpcMainEvent): Promise<void> {
    log.debug('handleStopCheck');

    if (!this.handle) {
      return;
    }

    clearInterval(this.handle);
    this.handle = undefined;
  }

  @IpcMainApi.Handle(IdleChannel.changeLimitTime)
  async handleChangeLimitTime(
    event: Electron.IpcMainEvent,
    limitTime: number
  ): Promise<void> {
    log.debug('handleChangeLimitTime');

    this.limitTimeInSecond = limitTime;
  }

  private idleCheck() {
    const idle: number = Electron.powerMonitor.getSystemIdleTime();
    if (idle > this.limitTimeInSecond) {
      if (WindowIdle.Active === this.idle) {
        this.idle = WindowIdle.Idle;
        BrowserWindowUtil.main().webContents.send(
          IdleChannel.onState$,
          this.idle
        );
      }
    } else {
      if (WindowIdle.Idle === this.idle) {
        this.idle = WindowIdle.Active;
        BrowserWindowUtil.main().webContents.send(
          IdleChannel.onState$,
          this.idle
        );
      }
    }
  }
}
